import { highlightAll } from 'prismjs';
import { isPlatformBrowser, CommonModule } from '@angular/common';
import { InjectionToken, Component, ViewEncapsulation, Inject, PLATFORM_ID, ElementRef, Input, NgModule, Injectable, Optional } from '@angular/core';
import { HttpResponse, HttpClient, HttpClientModule } from '@angular/common/http';
import marked from 'marked';
import { of, throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';

const NT_MARKDOWN_ENGINE = new InjectionToken('nt-markdown-engine');

class NtMarkdownComponent {
    constructor(_platformId, _markdownEngine, _elementRef) {
        this._platformId = _platformId;
        this._markdownEngine = _markdownEngine;
        this._elementRef = _elementRef;
    }
    get path() { return this._path; }
    set path(value) { this._path = value || ''; }
    get data() { return this._data; }
    set data(value) { this._data = value; }
    ngOnChanges(changes) {
        const pathChange = changes.path;
        const dataChange = changes.data;
        if (pathChange && !pathChange.firstChange) {
            this._pathChange();
        }
        if (dataChange && !dataChange.firstChange) {
            this._dataChange();
        }
    }
    ngAfterViewInit() {
        if (this.path) {
            this._pathChange();
        }
        else {
            this._dataChange();
        }
    }
    _dataChange() {
        if (this.data) {
            this._markdownEngine
                .compile(this.data)
                .subscribe(markdown => this._setMarkdownHtml(markdown));
        }
        else {
            this._setMarkdownHtml('');
        }
    }
    _pathChange() {
        if (this.path) {
            this._ext = this.path && this.path.split('.').splice(-1).join();
            this._markdownEngine.getContent(this.path).subscribe(content => {
                content = this._ext !== 'md' ? ['```', `${this._ext}\n${content}\n`, '```'].join() : content;
                this._markdownEngine.compile(this._prepare(content)).subscribe(markdown => this._setMarkdownHtml(markdown));
            }, error => this._handleError(error));
        }
    }
    _handleError(error) {
        return Promise.reject(error.message || error);
    }
    _prepare(raw) {
        if (!raw) {
            return '';
        }
        if (this._ext === 'md' || !this.path) {
            let isCodeBlock = false;
            return raw.split('\n').map((line) => {
                if (this._trimLeft(line).substring(0, 3) === "```") {
                    isCodeBlock = !isCodeBlock;
                }
                return isCodeBlock ? line : line.trim();
            }).join('\n');
        }
        return raw.replace(/\"/g, '\'');
    }
    _postpare() {
        if (isPlatformBrowser(this._platformId)) {
            highlightAll(false);
        }
    }
    _setMarkdownHtml(markdown) {
        this._elementRef.nativeElement.innerHTML = markdown;
        this._postpare();
    }
    _trimLeft(line) {
        return line.replace(/^\s+|\s+$/g, '');
    }
}
NtMarkdownComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-markdown, [nt-markdown]',
                template: '',
                encapsulation: ViewEncapsulation.None,
                host: {
                    class: 'nt-markdown'
                }
            },] }
];
NtMarkdownComponent.ctorParameters = () => [
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_MARKDOWN_ENGINE,] }] },
    { type: ElementRef }
];
NtMarkdownComponent.propDecorators = {
    path: [{ type: Input }],
    data: [{ type: Input }]
};

class NtMarkdownModule {
}
NtMarkdownModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [NtMarkdownComponent],
                exports: [NtMarkdownComponent]
            },] }
];

const NT_MARKED_DEFAULT_OPTIONS = {
    gfm: true,
    breaks: false,
    pedantic: false,
    smartLists: true,
    smartypants: false
};
const NT_MARKED_OPTIONS = new InjectionToken('nt-marked-options');

class NtMarkedEngine {
    constructor(http, options = {}) {
        this.http = http;
        this._renderer = new marked.Renderer();
        this.extendRenderer();
        this.setMarkedOptions(options);
    }
    get renderer() { return this._renderer; }
    getContent(path) {
        return this.http.get(path, { responseType: 'text' }).pipe(catchError(this.handleError));
    }
    setMarkedOptions(options) {
        options.renderer = this._renderer;
        marked.setOptions(options);
    }
    compile(data) { return of(marked(data)); }
    handleError(error) {
        let errMsg;
        if (error instanceof HttpResponse) {
            const body = error.body || '';
            const err = body.error || JSON.stringify(body);
            errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
        }
        else {
            errMsg = error.message ? error.message : error.toString();
        }
        return throwError(errMsg);
    }
    extendRenderer() {
        this._renderer.heading = function (text, level) {
            text = text.trim();
            return `<h${level} id="${text.replace(/\<a[^>]*?\>((.|\n)*?)\<\/a\>/g, '$1').replace(/ +/g, '-')}">
						${text}
					</h${level}>`;
        };
        this._renderer.listitem = function (text) {
            if (/^\s*\[[x ]\]\s*/.test(text)) {
                const checkedStyle = 'vertical-align: middle; margin: 0 0.2em 0.25em -1.6em; font-size: 16px;';
                text = text
                    .replace(/^\s*\[ \]\s*/, `<input type="checkbox" style="${checkedStyle}" disabled>`)
                    .replace(/^\s*\[x\]\s*/, `<input type="checkbox" style="${checkedStyle}" checked disabled>`);
                return `<li style="list-style: none">${text}</li>`;
            }
            else {
                return `<li>${text}</li>`;
            }
        };
    }
}
NtMarkedEngine.decorators = [
    { type: Injectable }
];
NtMarkedEngine.ctorParameters = () => [
    { type: HttpClient },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_MARKED_OPTIONS,] }] }
];

const ɵ0 = NT_MARKED_DEFAULT_OPTIONS;
class NtMarkedEngineModule {
}
NtMarkedEngineModule.decorators = [
    { type: NgModule, args: [{
                imports: [HttpClientModule],
                providers: [
                    { provide: NT_MARKDOWN_ENGINE, useClass: NtMarkedEngine },
                    { provide: NT_MARKED_OPTIONS, useValue: ɵ0 },
                ],
            },] }
];

export { NT_MARKDOWN_ENGINE, NT_MARKED_DEFAULT_OPTIONS, NT_MARKED_OPTIONS, NtMarkdownComponent, NtMarkdownModule, NtMarkedEngine, NtMarkedEngineModule, ɵ0 };
//# sourceMappingURL=markdown.js.map
